@@ -1,7 +1,7 @@
 /*
  * This file is part of the Shiboken Python Bindings Generator project.
  *
- * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
+ * Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
  *
  * Contact: PySide team <contact@pyside.org>
  *
@@ -203,9 +203,24 @@ SbkObjectType SbkObject_Type = { { {
 };
 
 
-void SbkDeallocWrapper(PyObject* pyObj)
+static void SbkDeallocWrapperCommon(PyObject* pyObj, bool canDelete)
 {
     SbkObject* sbkObj = reinterpret_cast<SbkObject*>(pyObj);
+    PyTypeObject* pyType = Py_TYPE(pyObj);
+
+    // Need to decref the type if this is the dealloc func; if type
+    // is subclassed, that dealloc func will decref (see subtype_dealloc
+    // in typeobject.c in the python sources)
+    bool needTypeDecref = (pyType->tp_dealloc == SbkDeallocWrapper
+                           || pyType->tp_dealloc == SbkDeallocWrapperWithPrivateDtor);
+
+    // Ensure that the GC is no longer tracking this object to avoid a
+    // possible reentrancy problem.  Since there are multiple steps involved
+    // in deallocating a SbkObject it is possible for the garbage collector to
+    // be invoked and it trying to delete this object while it is still in
+    // progress from the first time around, resulting in a double delete and a
+    // crash.
+    PyObject_GC_UnTrack(pyObj);
 
     // Check that Python is still initialized as sometimes this is called by a static destructor
     // after Python interpeter is shutdown.
@@ -213,8 +228,8 @@ void SbkDeallocWrapper(PyObject* pyObj)
         PyObject_ClearWeakRefs(pyObj);
 
     // If I have ownership and is valid delete C++ pointer
-    if (sbkObj->d->hasOwnership && sbkObj->d->validCppObject) {
-        SbkObjectType* sbkType = reinterpret_cast<SbkObjectType*>(pyObj->ob_type);
+    if (canDelete && sbkObj->d->hasOwnership && sbkObj->d->validCppObject) {
+        SbkObjectType* sbkType = reinterpret_cast<SbkObjectType*>(pyType);
         if (sbkType->d->is_multicpp) {
             Shiboken::DeallocVisitor visitor(sbkObj);
             Shiboken::walkThroughClassHierarchy(pyObj->ob_type, &visitor);
@@ -230,17 +245,19 @@ void SbkDeallocWrapper(PyObject* pyObj)
     } else {
         Shiboken::Object::deallocData(sbkObj, true);
     }
+
+    if (needTypeDecref)
+        Py_DECREF(pyType);
+}
+
+void SbkDeallocWrapper(PyObject* pyObj)
+{
+    SbkDeallocWrapperCommon(pyObj, true);
 }
 
 void SbkDeallocWrapperWithPrivateDtor(PyObject* self)
 {
-    SbkObject* sbkObj = reinterpret_cast<SbkObject*>(self);
-    // Check that Python is still initialized as sometimes this is called by a static destructor
-    // after Python interpeter is shutdown.
-    if (sbkObj->weakreflist && Py_IsInitialized())
-        PyObject_ClearWeakRefs(self);
-
-    Shiboken::Object::deallocData(sbkObj, true);
+    SbkDeallocWrapperCommon(self, false);
 }
 
 void SbkObjectTypeDealloc(PyObject* pyObj)
@@ -738,6 +755,8 @@ void setTypeUserData(SbkObjectType* self, void* userData, DeleteUserDataFunc d_f
 namespace Object
 {
 
+static void recursive_invalidate(SbkObject* self, std::set<SbkObject*>& seen);
+
 bool checkType(PyObject* pyObj)
 {
     return ObjectType::checkType(pyObj->ob_type);
@@ -748,6 +767,12 @@ bool isUserType(PyObject* pyObj)
     return ObjectType::isUserType(pyObj->ob_type);
 }
 
+Py_hash_t hash(PyObject* pyObj)
+{
+    assert(Shiboken::Object::checkType(pyObj));
+    return reinterpret_cast<Py_hash_t>(pyObj);
+}
+
 static void setSequenceOwnership(PyObject* pyObj, bool owner)
 {
     if (PySequence_Check(pyObj)) {
@@ -798,9 +823,19 @@ void callCppDestructors(SbkObject* pyObj)
         threadSaver.save();
         sbkType->d->cpp_dtor(pyObj->d->cptr[0]);
     }
+
+    /* invalidate needs to be called before deleting pointer array because
+       it needs to delete entries for them from the BindingManager hash table;
+       also release wrapper explicitly if object contains C++ wrapper because
+       invalidate doesn't */
+    invalidate(pyObj);
+    if (pyObj->d->validCppObject && pyObj->d->containsCppWrapper) {
+      BindingManager::instance().releaseWrapper(pyObj);
+    }
+
     delete[] pyObj->d->cptr;
     pyObj->d->cptr = 0;
-    invalidate(pyObj);
+    pyObj->d->validCppObject = false;
 }
 
 bool hasOwnership(SbkObject* pyObj)
@@ -855,19 +890,36 @@ void releaseOwnership(PyObject* self)
     setSequenceOwnership(self, false);
 }
 
+/* Needed forward declarations */
+static void recursive_invalidate(PyObject* pyobj, std::set<SbkObject*>& seen);
+static void recursive_invalidate(SbkObject* self, std::set<SbkObject*>& seen);
+
 void invalidate(PyObject* pyobj)
 {
-    std::list<SbkObject*> objs = splitPyObject(pyobj);
-    std::list<SbkObject*>::const_iterator it = objs.begin();
-    for(; it != objs.end(); it++)
-        invalidate(*it);
+    std::set<SbkObject*> seen;
+    recursive_invalidate(pyobj, seen);
 }
 
 void invalidate(SbkObject* self)
 {
-    // Skip if this object not is a valid object
-    if (!self || ((PyObject*)self == Py_None))
+    std::set<SbkObject*> seen;
+    recursive_invalidate(self, seen);
+}
+
+static void recursive_invalidate(PyObject* pyobj, std::set<SbkObject*>& seen)
+{
+    std::list<SbkObject*> objs = splitPyObject(pyobj);
+    std::list<SbkObject*>::const_iterator it = objs.begin();
+    for (; it != objs.end(); it++)
+        recursive_invalidate(*it, seen);
+}
+
+static void recursive_invalidate(SbkObject* self, std::set<SbkObject*>& seen)
+{
+    // Skip if this object not is a valid object or if it's already been seen
+    if (!self || ((PyObject*)self == Py_None) || seen.find(self) != seen.end())
         return;
+    seen.insert(self);
 
     if (!self->d->containsCppWrapper) {
         self->d->validCppObject = false; // Mark object as invalid only if this is not a wrapper class
@@ -882,7 +934,7 @@ void invalidate(SbkObject* self)
 
         for (; it != copy.end(); ++it) {
             // invalidate the child
-            invalidate(*it);
+            recursive_invalidate(*it, seen);
 
             // if the parent not is a wrapper class, then remove children from him, because We do not know when this object will be destroyed
             if (!self->d->validCppObject)
@@ -898,7 +950,7 @@ void invalidate(SbkObject* self)
             const std::list<PyObject*> lst = iter->second;
             std::list<PyObject*>::const_iterator it = lst.begin();
             while(it != lst.end()) {
-                invalidate(*it);
+                recursive_invalidate(*it, seen);
                 ++it;
             }
         }
@@ -1315,16 +1367,21 @@ std::string info(SbkObject* self)
     std::ostringstream s;
     std::list<SbkObjectType*> bases;
 
-    if (ObjectType::isUserType(Py_TYPE(self)))
-        bases = getCppBaseClasses(Py_TYPE(self));
-    else
-        bases.push_back(reinterpret_cast<SbkObjectType*>(Py_TYPE(self)));
+    if (self->d && self->d->cptr) {
+        if (ObjectType::isUserType(Py_TYPE(self)))
+            bases = getCppBaseClasses(Py_TYPE(self));
+        else
+            bases.push_back(reinterpret_cast<SbkObjectType*>(Py_TYPE(self)));
 
-    s << "C++ address....... ";
-    std::list<SbkObjectType*>::const_iterator it = bases.begin();
-    for (int i = 0; it != bases.end(); ++it, ++i)
-        s << ((PyTypeObject*)*it)->tp_name << "/" << self->d->cptr[i] << ' ';
-    s << "\n";
+        s << "C++ address....... ";
+        std::list<SbkObjectType*>::const_iterator it = bases.begin();
+        for (int i = 0; it != bases.end(); ++it, ++i)
+            s << ((PyTypeObject*)*it)->tp_name << "/" << self->d->cptr[i] << ' ';
+        s << "\n";
+    }
+    else {
+        s << "C++ address....... <<Deleted>>\n";
+    }
 
     s << "hasOwnership...... " << bool(self->d->hasOwnership) << "\n"
          "containsCppWrapper " << self->d->containsCppWrapper << "\n"
