@@ -1,7 +1,7 @@
 /*
  * This file is part of the Shiboken Python Bindings Generator project.
  *
- * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+ * Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
  *
  * Contact: PySide team <contact@pyside.org>
  *
@@ -27,11 +27,11 @@
 #include "sbkdbg.h"
 #include "gilstate.h"
 #include "sbkstring.h"
+#include "debugfreehook.h"
 
 #include <cstddef>
 #include <fstream>
 
-
 namespace Shiboken
 {
 
@@ -150,20 +150,29 @@ BindingManager::BindingManager()
     m_d = new BindingManager::BindingManagerPrivate;
     m_d->wrapperMapper.set_empty_key((WrapperMap::key_type)0);
     m_d->wrapperMapper.set_deleted_key((WrapperMap::key_type)1);
+
+#ifdef SHIBOKEN_INSTALL_FREE_DEBUG_HOOK
+    debugInstallFreeHook();
+#endif
 }
 
 BindingManager::~BindingManager()
 {
+#ifdef SHIBOKEN_INSTALL_FREE_DEBUG_HOOK
+    debugRemoveFreeHook();
+#endif
 #ifndef NDEBUG
     showWrapperMap(m_d->wrapperMapper);
 #endif
     /* Cleanup hanging references. We just invalidate them as when
      * the BindingManager is being destroyed the interpreter is alredy
      * shutting down. */
-    while (!m_d->wrapperMapper.empty()) {
-        Object::destroy(m_d->wrapperMapper.begin()->second, const_cast<void*>(m_d->wrapperMapper.begin()->first));
+    if (Py_IsInitialized()) {  // ensure the interpreter is still valid
+        while (!m_d->wrapperMapper.empty()) {
+            Object::destroy(m_d->wrapperMapper.begin()->second, const_cast<void*>(m_d->wrapperMapper.begin()->first));
+        }
+        assert(m_d->wrapperMapper.size() == 0);
     }
-    assert(m_d->wrapperMapper.size() == 0);
     delete m_d;
 }
 
