@@ -1,7 +1,7 @@
 /*
  * This file is part of the Shiboken Python Binding Generator project.
  *
- * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+ * Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
  *
  * Contact: PySide team <contact@pyside.org>
  *
@@ -131,6 +131,18 @@ doubleUnsignedInt(unsigned int value)
     return value * 2;
 }
 
+long long
+doubleLongLong(long long value)
+{
+    return value * 2;
+}
+
+unsigned long long
+doubleUnsignedLongLong(unsigned long long value)
+{
+    return value * 2;
+}
+
 short
 doubleShort(short value)
 {
