@@ -1,7 +1,7 @@
 /*
  * This file is part of the Shiboken Python Binding Generator project.
  *
- * Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
+ * Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
  *
  * Contact: PySide team <contact@pyside.org>
  *
@@ -28,11 +28,11 @@
 class LIBSAMPLE_API SbkDate
 {
 public:
-    SbkDate(int d, int m, int y) : m_d(d), m_m(m), m_y(y) {}
+    SbkDate(int d, int m, int y);
 
-    inline int day() const { return m_d; }
-    inline int month() const { return m_m; }
-    inline int year() const { return m_y; }
+    int day() const;
+    int month() const;
+    int year() const;
 
 private:
     int m_d;
