@@ -3,7 +3,7 @@
 #
 # This file is part of the Shiboken Python Bindings Generator project.
 #
-# Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
+# Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 #
 # Contact: PySide team <contact@pyside.org>
 #
@@ -21,17 +21,17 @@
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-# 02110-1301 USA
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
-'''Test case for a class that holds a unknown handle object.'''
+''' Test case for a class that holds a unknown handle object.
+    Test case for BUG #1105.
+'''
 
 import unittest
 
 from sample import HandleHolder
 
 class HandleHolderTest(unittest.TestCase):
-
     def testCreation(self):
         holder = HandleHolder(HandleHolder.createHandle())
         holder2 = HandleHolder(HandleHolder.createHandle())
@@ -39,9 +39,13 @@ class HandleHolderTest(unittest.TestCase):
 
     def testTransfer(self):
         holder = HandleHolder()
-        holder2 = HandleHolder(holder.get())
+        holder2 = HandleHolder(holder.handle())
         self.assertTrue(holder.compare(holder2))
 
+    def testUseDefinedType(self):
+        holder = HandleHolder(8)
+        holder2 = HandleHolder(holder.handle2())
+        self.assertTrue(holder.compare2(holder2))
+
 if __name__ == '__main__':
     unittest.main()
-
